//
// ColorPickerWnd.cpp
//

#include "ColorPickerWnd.h"

const uint32 MSG_COLOR_CHANGED	= 'COLO';
const uint32 MSG_BTN_CANCEL		= 'CANC';
const uint32 MSG_BTN_DEFAULT	= 'DEFA';

// Constructor...
//
// Args:
//	* Title: 			Titel of the window.
//	* pCrSelected:		Pointer to the color that should be changed
//
CColorPickerWnd::CColorPickerWnd(const char* title, BRect rectScreen, rgb_color* pCrSelected)
	:BWindow(BRect(
		rectScreen.right / 2 - 140, rectScreen.bottom / 2 - 50,		// center window
		rectScreen.right / 2 + 140, rectScreen.bottom / 2 + 50), 
	title, B_FLOATING_WINDOW,
	B_ASYNCHRONOUS_CONTROLS|B_NOT_ZOOMABLE|B_NOT_RESIZABLE)
{
	m_bCloseRequested = false;
	m_pCrSelected = pCrSelected;
	
	// Add ColorControl...
	AddChild(m_pColorCtrl = new BColorControl(BPoint(5, 5), B_CELLS_32x8, 4,
			NULL, new BMessage(MSG_COLOR_CHANGED), true));
	m_pColorCtrl->SetValue(*m_pCrSelected);

	// Add Buttons...
	AddChild(m_pBtnOk = new BButton(BRect(200, 70, 270, 85), "OkBtn", "Ok", new BMessage(B_QUIT_REQUESTED)));
	m_pBtnOk->MakeDefault(true);	// make "Ok" button to default button
	
	AddChild(m_pBtnCancel = new BButton(BRect(120, 70, 190, 85), "CancelBtn", "Cancel", new BMessage(MSG_BTN_CANCEL)));

	AddChild(m_pBtnDefault = new BButton(BRect(40, 70, 110, 85), "DefaultBtn", "Default", new BMessage(MSG_BTN_DEFAULT)));
}

// Show...
void CColorPickerWnd::Show()
{
	BWindow::Show();
	m_crDefault = *m_pCrSelected;
}

// MessageReceived...
void CColorPickerWnd::MessageReceived(BMessage* pMsg)
{
	switch(pMsg->what)
	{
		case MSG_COLOR_CHANGED:								// Color changed?
			*m_pCrSelected = m_pColorCtrl->ValueAsColor();	// Get color
				m_pMainWnd->PostMessage(MSG_REFRESH_HTML);
			break;
		case MSG_BTN_CANCEL:								// Cancel button?
			RestoreDefaultColor();
			Hide();											// Hide window
			break;
		case MSG_BTN_DEFAULT:								// Default color?
			RestoreDefaultColor();
			break;
		default:
			BWindow::MessageReceived(pMsg);
	}
}

// QuitRequested...
// When the member variable m_bCloseRequested is true, the window will be closed, else
// it will be only hidden.
bool CColorPickerWnd::QuitRequested()
{
	if(m_bCloseRequested == false)
		Hide();				// just hide the window

	return m_bCloseRequested;
}

// RestoreDefaultColor...
//
// restores the default color
void CColorPickerWnd::RestoreDefaultColor()
{
	*m_pCrSelected = m_crDefault;			// Restore color in ColorTextView
	m_pColorCtrl->SetValue(m_crDefault);	// Restore color in Color-Control
	m_pMainWnd->PostMessage(MSG_REFRESH_HTML);
}