//
// MainWnd.cpp
//

#include "MainWnd.h"

const uint32 MSG_FILE_PREVIEW			= 'BROW';
const uint32 MSG_FILE_HELP				= 'HELP';
const uint32 MSG_EDIT_COLOR_START		= 'CRST';
const uint32 MSG_EDIT_COLOR_END			= 'CREN';
const uint32 MSG_EDIT_SWAP_COLORS		= 'SWAP';
const uint32 MSG_EDIT_WRAP				= 'WRAP';
const uint32 MSG_EDIT_UPPERCASE_TAGS	= 'UPPE';
const uint32 MSG_EDIT_QUOTATION_MARKS 	= 'QUOT';

const char SETTING_DIR[] = "muramoto";
const char SETTING_FILE[] = "ColorFont2Html";
const char SETTING_RECT[] = "Position";
const char SETTING_STARTCOLOR[] = "StartColor";
const char SETTING_ENDCOLOR[] = "EndColor";
const char SETTING_WRAP[] = "Wrap";
const char SETTING_UPPERCASE_TAGS[] = "UppercaseTags";
const char SETTING_QUOTATION_MARKS[] = "QuotationMarks";

// Constructor...
CMainWnd::CMainWnd(BRect rectScreen)
	:BWindow(BRect(
		rectScreen.right / 2 - 200, rectScreen.bottom / 2 - 125,		// center window
		rectScreen.right / 2 + 200, rectScreen.bottom / 2 + 125), 
	"ColorFont2Html", B_DOCUMENT_WINDOW, B_ASYNCHRONOUS_CONTROLS)
{			
	// Menues...
	m_pMenuBar = new BMenuBar(BRect(0, 0, 0, 0), "Menubar");

	// File menue...
	m_pMenuFile = new BMenu("File");
	m_pMenuFile->AddItem(m_pMenuFilePreview = new BMenuItem("Preview in browser...", new BMessage(MSG_FILE_PREVIEW), 'P'));
	m_pMenuFile->AddSeparatorItem();
	
	m_pMenuFile->AddItem(m_pMenuFileHelp = new BMenuItem("Help...", new BMessage(MSG_FILE_HELP), 'H'));	
	m_pMenuFile->AddItem(m_pMenuFileAbout = new BMenuItem("About...", new BMessage(B_ABOUT_REQUESTED), 'A'));
	m_pMenuFileAbout->SetTarget(be_app);
	m_pMenuFile->AddSeparatorItem();
	
	m_pMenuFile->AddItem(m_pMenuFileQuit = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q'));
	m_pMenuFileQuit->SetTarget(be_app);

	m_pMenuBar->AddItem(m_pMenuFile);		// add file menu to the menubar

	// Edit menue...
	m_pMenuEdit = new BMenu("Edit");
	m_pMenuEdit->AddItem(m_pMenuEditStartColor = new BMenuItem("Edit start color...", new BMessage(MSG_EDIT_COLOR_START), 'S'));
	m_pMenuEdit->AddItem(m_pMenuEditEndColor = new BMenuItem("Edit end color...", new BMessage(MSG_EDIT_COLOR_END), 'E'));
	m_pMenuEdit->AddItem(m_pMenuEditSwapColors = new BMenuItem("Swap start/end color", new BMessage(MSG_EDIT_SWAP_COLORS), 'R'));
	m_pMenuEdit->AddSeparatorItem();
	
	m_pMenuEdit->AddItem(m_pMenuEditCopy = new BMenuItem("Copy HTML-Sourcecode", new BMessage(B_COPY), 'C'));		
	m_pMenuEdit->AddSeparatorItem();

	m_pMenuEdit->AddItem(m_pMenuEditWrap = new BMenuItem("Wrap line after space?", new BMessage(MSG_EDIT_WRAP), 'W'));
	m_pMenuEditWrap->SetMarked(true);		// check it!
	m_pMenuEdit->AddItem(m_pMenuEditUppercaseTags = new BMenuItem("UPPERCASE <Tags>?", new BMessage(MSG_EDIT_UPPERCASE_TAGS), 'U'));
	m_pMenuEdit->AddItem(m_pMenuEditQuotationMarks = new BMenuItem("Use \"quotations\" marks?", new BMessage(MSG_EDIT_QUOTATION_MARKS), '\"'));
	
	m_pMenuBar->AddItem(m_pMenuEdit);		// add edit menu to the menubar
	
	AddChild(m_pMenuBar);					// add menu to the window
		
	// ColorTextView and ScrollView...
	BRect rectMenuBar = m_pMenuBar->Bounds();
	BRect rectFrame(0, rectMenuBar.bottom + 1, rectMenuBar.right, rectMenuBar.bottom + 45);
	
	m_pColorTextView = new CColorTextView(rectFrame);

	// Set Startcolors in the ColorTextView
	const rgb_color crRed = {255, 0, 0, 0};
	const rgb_color crBlue = {0, 0, 255, 0};	
	m_pColorTextView->m_crStart = crRed;
	m_pColorTextView->m_crEnd = crBlue;
	
	AddChild(m_pScrollViewColor = new BScrollView("ScrollViewColor", m_pColorTextView,
			B_FOLLOW_LEFT_RIGHT, 0, true, false));	
	
	// HtmlTextView and ScrollView...
	rectFrame.OffsetTo(0, rectFrame.bottom + B_H_SCROLL_BAR_HEIGHT + 5);
	rectFrame.bottom = Bounds().bottom - B_H_SCROLL_BAR_HEIGHT;
	rectFrame.right = rectFrame.right - B_V_SCROLL_BAR_WIDTH;
	BRect rectText = rectFrame;
	rectText.OffsetTo(B_ORIGIN);
	
	m_pTextViewHtml = new BTextView(rectFrame, "TextViewHtml", rectText,
			B_FOLLOW_ALL, B_WILL_DRAW);
	m_pTextViewHtml->MakeEditable(false);	// disallow edit in Html-TextView

	AddChild(m_pScrollViewHtml = new BScrollView("ScrollViewHtml", m_pTextViewHtml,
			B_FOLLOW_ALL, 0, true, true));

	// Set attributes of the Color-TextView...
	m_pColorTextView->m_pTextViewHtml = m_pTextViewHtml;
	m_pColorTextView->m_bWrapLine = true;
	m_pColorTextView->SetText("Enter your text here!");
	m_pColorTextView->MakeFocus(true);		// set focus to the TextView
	m_pColorTextView->SelectAll();			// select the text
		
	// Last but not least, we create two ColorPicker-windows for the start- and 
	// endcolor...
	// I show the window and the hide it, otherwise the application produce an error
	// when I call Show().
	// If anybody knows the reason, please mail to me: bernhardberger@yahoo.de
	//
	(m_pColorPickerWndStart = new CColorPickerWnd("Edit start color", rectScreen,
			&(m_pColorTextView->m_crStart)))->Show();
			
	m_pColorPickerWndStart->Hide();	// hide window

	(m_pColorPickerWndEnd = new CColorPickerWnd("Edit end color", rectScreen,
			&(m_pColorTextView->m_crEnd)))->Show();
	m_pColorPickerWndEnd->Hide();	// hide window

	//Restore previous settings...
	RestoreSettings();
}

// MessageReceived...
void CMainWnd::MessageReceived(BMessage* pMsg)
{
	switch(pMsg->what)
	{
		case MSG_REFRESH_HTML:				// Refresh HTML sourcecode?
			// the ColorPickerWnd sends this message, when the color is changed, and
			// so the HTML sourcecode has to be refreshed.
			m_pColorTextView->RefreshHtml();
			break;
		
		case MSG_FILE_PREVIEW:				// Preview in browser?
			PreviewInBrowser();
			break;
		case MSG_FILE_HELP:					// Help?
			LaunchApplication(GetFullFilename("Help.html"));	// Show help
			break;
		case MSG_EDIT_COLOR_START:			// edit start color?
			if(m_pColorPickerWndStart->IsHidden())
				m_pColorPickerWndStart->Show();	// show Start-Color window
			break;
		case MSG_EDIT_COLOR_END:			// edit end color?
			if(m_pColorPickerWndEnd->IsHidden())
				m_pColorPickerWndEnd->Show();	// show End-Color window
			break;
		case MSG_EDIT_SWAP_COLORS:			// swap start- and end color?
			m_pColorTextView->SwapColors();
			break;
		case B_COPY:						// Copy HTML-sourcecode to clipboard
			CopyHtmlSource2Clipboard();
			break;
		case MSG_EDIT_WRAP:					// Wrap line after space in HTML sourcecode?
			// Set/Unset menu item...
			m_pMenuEditWrap->SetMarked(m_pColorTextView->m_bWrapLine = !m_pColorTextView->m_bWrapLine);
			
			m_pColorTextView->RefreshHtml();  // Refresh the HTML source code
			break;
		case MSG_EDIT_UPPERCASE_TAGS:		// UPPERCASE <Tags>?
			// Set/Unset menu item...
			m_pMenuEditUppercaseTags->SetMarked(m_pColorTextView->m_bUppercaseTags = !m_pColorTextView->m_bUppercaseTags);
						
			m_pColorTextView->RefreshHtml();  // Refresh the HTML source code
			break;					 
		case MSG_EDIT_QUOTATION_MARKS:		// Use "quotation" marks?
			// Set/Unset menu item...
			m_pMenuEditQuotationMarks->SetMarked(m_pColorTextView->m_bQuotationMarks = !m_pColorTextView->m_bQuotationMarks);
						
			m_pColorTextView->RefreshHtml();  // Refresh the HTML source code
			break;					 
		default:							// other message
			BWindow::MessageReceived(pMsg);
	}
}

// QuitRequested...
bool CMainWnd::QuitRequested()
{
	SaveSettings();

	// We want to close all windows...
	m_pColorPickerWndStart->m_bCloseRequested = true;
	m_pColorPickerWndEnd->m_bCloseRequested = true;

	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;		// close window
}

// FrameResized...
void CMainWnd::FrameResized(float width, float height)
{
	BRect textrect = m_pTextViewHtml->TextRect();
	
	textrect.right = textrect.left + (width - B_V_SCROLL_BAR_WIDTH);
	m_pTextViewHtml->SetTextRect(textrect);
}

//
// *** MENUE ROUTINES ***
//

// PreviewInBrowser
//
// writes the HTML sourcecode to a temporary file (Preview.html) and launch it.
void CMainWnd::PreviewInBrowser()
{
	const char* strTempName = GetFullFilename("Preview.html");
	BFile fileTemp;
	
	// Create new file...
	if(fileTemp.SetTo(strTempName, B_CREATE_FILE|B_WRITE_ONLY|B_ERASE_FILE) == B_OK)
	{
		const char* strHtmlHead = "<HTML>\n<HEAD>\n"
			"<!-- This is a temporary preview file made from the application ColorFont2Html.\n"
			"When you want, you can delete this file. -->\n"
			"<TITLE>ColorFont2Html-Preview</TITLE>\n</HEAD>\n"
			"<BODY>\n<H1 align=\"center\">\n";
		const char*	strHtmlTail = "\n</H1>\n</BODY>\n</HTML>";				
		
		fileTemp.Write(strHtmlHead, strlen(strHtmlHead));		// write Html header
		
		fileTemp.Write(m_pColorTextView->m_strHtml.String(), 
				m_pColorTextView->m_strHtml.Length());			// write Html body
		
		fileTemp.Write(strHtmlTail, strlen(strHtmlTail));		// write Html tail
	
		LaunchApplication(strTempName);							// start browser
	}
	else			// file error?
	{
		// show the error message...
		(new BAlert(NULL, "Sorry, I cannot write the temporary HTML-Preview-File.", 
					"Oh!", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
	}	
}

// CopyHtmlSource2Clipboard
//
// Copies the html sourcecode to the clipboard...
void CMainWnd::CopyHtmlSource2Clipboard()
{
	if(be_clipboard->Lock())				// lock clipboard
	{
		be_clipboard->Clear();				// delete old clipboard

		BMessage* msgClip = (BMessage*)NULL;
		if((msgClip = be_clipboard->Data()))
		{
			// Set HTML-Sourcecode to the clipboard...
			msgClip->AddData("text/plain", B_MIME_TYPE, 
							m_pColorTextView->m_strHtml.String(), 
							m_pColorTextView->m_strHtml.Length());
							
			be_clipboard->Commit();			// send the datas
		}
		be_clipboard->Unlock();				// unlock clipboard
	}
}

//
// *** HELP ROUTINES ***
//

// GetFullFilename...
//
// Retrieves the full-filename of files in the app directory. 
const char* CMainWnd::GetFullFilename(const char* strFilename)
{
	// First, we get the path from where the application is launched...
	app_info info;
	if (be_app->GetAppInfo(&info) == B_OK)	// Get info about the application
	{
		BEntry entry = BEntry(&info.ref);
		entry.GetPath(&m_pathApp);
		m_pathApp.GetParent(&m_pathApp);
	}
	else
		m_pathApp = BPath(".");

	m_pathApp.Append(strFilename);

	// The member variable m_pathApp will only be used by this routine. (When we
	// would use a local object, the BPath instance will get delete, and so we cannot
	// return the path).
	return m_pathApp.Path();
}

// LaunchApplication...
//
// launch an application
status_t CMainWnd::LaunchApplication(const char* strAppName)
{
	status_t	Result;
	entry_ref	Ref;
	BEntry		entryApp(strAppName);		// generate BEntry for app
			
	entryApp.GetRef(&Ref);					// search Ref for app
	Result = be_roster->Launch(&Ref);		// launch app
	
	if((Result != B_NO_ERROR) && (Result != B_ALREADY_RUNNING))	// Error?
	{
		// Show the error message...
		char		string[512];
		sprintf(string, "Error launching file: %s\n(Filename: %s)", strerror(Result), strAppName);
		(new BAlert("", string, "Ups!", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
	}
	
	return Result;
}

//
// *** SETTING ROUTINES ***
//
// Thanks to Muramoto for these piece of code.
//

// GetSettingFile...
//
// Returns the BEntry of our setting file.
BEntry CMainWnd::GetSettingFile()
{
	// Get the path of our setting file...
	BPath pathSettingFile;
	::find_directory(B_USER_SETTINGS_DIRECTORY, &pathSettingFile, true);
	pathSettingFile.Append(SETTING_DIR);
	
	// If the directory doesn't exits, it will be created...
	BDirectory dummy;
	dummy.CreateDirectory(pathSettingFile.Path(), NULL);
	
	// Finally return the BEntry with the full path to our setting file...
	pathSettingFile.Append(SETTING_FILE);
	return BEntry(pathSettingFile.Path(), true);
}

// RestoreSettings...
//
// Reads and restores the settings from our setting file.
void CMainWnd::RestoreSettings()
{
	BEntry entrySetting = this->GetSettingFile();  // get our setting file
	
	// Read the setting file...
	BFile file;
	if(entrySetting.Exists())
	{
		file.SetTo(&entrySetting, B_READ_ONLY);
		if(file.InitCheck() != B_OK) return;
		
		// Window position...
		BRect rect;
		ssize_t size = file.ReadAttr(SETTING_RECT, B_RECT_TYPE, 0, &rect, sizeof(BRect));
		if(size == sizeof(BRect))
		{
			this->MoveTo(rect.LeftTop());
			this->ResizeTo(rect.Width(), rect.Height());
		}

		// Maybe the window isn't visible anymore (changed screen-resolution etc.).
		// In this case we move the window again...
		//
		// TODO: - check, if the window is really _full_ outside the screen
		//		 - add the titlebar to the new position
		BScreen screen(this);
		BRect screenFrame = screen.Frame();
		if(screenFrame.Contains(this->Frame()) == false)
			this->MoveTo(B_ORIGIN);
				
		// Color setting...
		rgb_color crStart;
		size = file.ReadAttr(SETTING_STARTCOLOR, B_RGB_COLOR_TYPE, 0, &crStart, sizeof(rgb_color));
		if(size == sizeof(rgb_color))
			m_pColorTextView->m_crStart = crStart;
		
		rgb_color crEnd;
		size = file.ReadAttr(SETTING_ENDCOLOR, B_RGB_COLOR_TYPE, 0, &crEnd, sizeof(rgb_color));
		if(size == sizeof(rgb_color))
			m_pColorTextView->m_crEnd = crEnd;
		
		// Html settings...
		// All the html settings were represented in check menues.
		// So we have to call the menu message to check/uncheck the menu.
		bool bWrapLine;
		size = file.ReadAttr(SETTING_WRAP, B_BOOL_TYPE, 0, &bWrapLine, sizeof(bool));
		if(size == sizeof(bool))
		{
			// WrapLine is default checked. So only if WrapLine was stored unset in the
			// settingfile, we have to post the message...
			if(bWrapLine == false) PostMessage(MSG_EDIT_WRAP);
		}

		bool bUppercaseTags;
		size = file.ReadAttr(SETTING_UPPERCASE_TAGS, B_BOOL_TYPE, 0, &bUppercaseTags, sizeof(bool));
		if(size == sizeof(bool))
		{
			if(bUppercaseTags) PostMessage(MSG_EDIT_UPPERCASE_TAGS);
		}
		
		bool bQuotationMarks;
		size = file.ReadAttr(SETTING_QUOTATION_MARKS, B_BOOL_TYPE, 0, &bQuotationMarks, sizeof(bool));
		if(size == sizeof(bool))
		{
			if(bQuotationMarks) PostMessage(MSG_EDIT_QUOTATION_MARKS);
		}
				
		m_pColorTextView->RefreshHtml();		// refresh the HTML			
	}

}

// SaveSettings....
//
// Write the settings to our setup file.
bool CMainWnd::SaveSettings()
{
	BEntry entrySetting = this->GetSettingFile();	// get setting file

	// After the first program was launched the first time, we have to create a
	// new file, else we just open it in the B_READ_WRITE-mode.
	BFile file;	
	file.SetTo(&entrySetting, (entrySetting.Exists()) ? B_READ_WRITE : B_CREATE_FILE|B_READ_WRITE|B_FAIL_IF_EXISTS);
	if(file.InitCheck() != B_OK) return false;
	
	// Save window position...
	BRect rect = this->Frame();
	file.WriteAttr(SETTING_RECT, B_RECT_TYPE, 0, &rect, sizeof(BRect));
	
	// Save color settings...
	rgb_color crStart = m_pColorTextView->m_crStart;
	file.WriteAttr(SETTING_STARTCOLOR, B_RGB_COLOR_TYPE, 0, &crStart, sizeof(rgb_color));
	
	rgb_color crEnd = m_pColorTextView->m_crEnd;
	file.WriteAttr(SETTING_ENDCOLOR, B_RGB_COLOR_TYPE, 0, &crEnd, sizeof(rgb_color));
	
	// Save html settings...
	bool bWrapLine = m_pColorTextView -> m_bWrapLine;
	file.WriteAttr(SETTING_WRAP, B_BOOL_TYPE, 0, &bWrapLine, sizeof(bool));

	bool bUppercaseTags = m_pColorTextView -> m_bUppercaseTags;
	file.WriteAttr(SETTING_UPPERCASE_TAGS, B_BOOL_TYPE, 0, &bUppercaseTags, sizeof(bool));

	bool bQuotationMarks = m_pColorTextView -> m_bQuotationMarks;
	file.WriteAttr(SETTING_QUOTATION_MARKS, B_BOOL_TYPE, 0, &bQuotationMarks, sizeof(bool));
	
	return true;
}
